const PROGRESS_UPDATE_INTERVAL = 50;

document.addEventListener('DOMContentLoaded', () => {
    const pages = Array.from(document.querySelectorAll('img.viewer-image'));
    let currentPage = parseInt(localStorage.getItem(`${WORK_ID}-currentPage`)) || 0;
    let duration = parseInt(localStorage.getItem(`${WORK_ID}-duration`)) || 10;
    let paused = true;
    let interval;
    let elapsed = 0;
    let rtl = (localStorage.getItem(`${WORK_ID}-rtl`) !== "false");

    function startTimer() {
        if (interval) {
            clearInterval(interval);
        }
        interval = setInterval(
            function () {
                if (paused) {
                    return;
                }
                elapsed += PROGRESS_UPDATE_INTERVAL;
                if (elapsed >= duration*1000) {
                    changePage(currentPage + 1);
                }
                updateBar();
            },
            PROGRESS_UPDATE_INTERVAL,
        );
    }

    const progressBar = document.getElementById('progress');
    function updateBar() {
        progressBar.style.width = `${100*elapsed/(1000*duration)}%`;
    }

    function stopTimer() {
        if (interval) {
            clearInterval(interval);
            interval = null;
        }
        elapsed = 0;
        updateBar();
    }

    function changePage(pageNum) {
        elapsed = 0;

        const previous = pages[currentPage];
        const current = pages[pageNum];

        if (current == null) {
            return;
        }

        previous.classList.remove('current');
        current.classList.add('current');

        currentPage = pageNum;
        localStorage.setItem(`${WORK_ID}-currentPage`, currentPage);

        const display = document.getElementById('image-container');
        display.style.backgroundImage = `url("${current.src}")`;

        document.getElementById('current-page').innerText = (pageNum + 1).toLocaleString();
    }

    const durationDisplay = document.getElementById('duration');
    function changeDuration(secs, pause) {
        duration = secs;
        localStorage.setItem(`${WORK_ID}-duration`, duration);
        paused = pause;

        durationDisplay.textContent = duration.toLocaleString() + 's';
        if (paused) {
            durationDisplay.style.textDecoration = "line-through";
            stopTimer();
        } else {
            durationDisplay.style.textDecoration = "";
            startTimer();
        }
    }

    function left() {
        if (currentPage === 0) {
            rtl = true;
            localStorage.setItem(`${WORK_ID}-rtl`, rtl);
        }
        changePage(currentPage + (rtl ? 1 : -1));
    }

    function right() {
        if (currentPage === 0) {
            rtl = false;
            localStorage.setItem(`${WORK_ID}-rtl`, rtl);
        }
        changePage(currentPage + (rtl ? -1 : 1));
    }

    function restart() {
        changePage(0);
    }

    function exitToWork() {
        changeDuration(duration, true);
        if (currentPage === pages.length - 1) {
            localStorage.removeItem(`${WORK_ID}-currentPage`);
        }
        window.location.href = `../${INDEX}`;
    }

    function hideTapZones() {
        document.getElementById('controls').style.opacity = 0;
    }

    changePage(currentPage);
    changeDuration(duration, paused);

    document.onkeydown = event => {
        hideTapZones();

        switch (event.key) {
        case "ArrowLeft":
            event.preventDefault();
            left();
            break;
        case "ArrowRight":
            event.preventDefault();
            right();
            break;
        case " ":
            event.preventDefault();
            changeDuration(duration, !paused);
            break;
        case "ArrowUp":
            event.preventDefault();
            if (2 <= duration && duration <= 10) {
                changeDuration(duration - 1, false);
            } else if (10 < duration && duration <= 20) {
                changeDuration(duration - 2.5, false);
            } else if (20 < duration) {
                changeDuration(duration - 5, false);
            }
            break;
        case "ArrowDown":
            event.preventDefault();
            if (duration < 10) {
                changeDuration(duration + 1, false);
            } else if (10 <= duration && duration < 20) {
                changeDuration(duration + 2.5, false);
            } else if (20 <= duration) {
                changeDuration(duration + 5, false);
            }
            break;
        case "Enter":
            event.preventDefault();
            restart();
            break;
        case "Escape":
            event.preventDefault();
            exitToWork();
            break;
        }
    };

    document.onclick = hideTapZones;
    document.getElementById("tap-left").onclick = left;
    document.getElementById("tap-right").onclick = right;
    document.getElementById("tap-restart").onclick = restart;
    document.getElementById("tap-back").onclick = exitToWork;
});
